function gtSetup()
% GTSETUP  Sets up the parameters file for the experiment
%
%     gtSetup()
%     ---------
%     To be run in the directory where the data has been collected
%
%     Version 012 14-06-2012 by LNervo
%       Add functions to set default parameters for seg, index, fsim and rec
%
%     Version 011 14-05-2012 by LNervo
%       Modify gtReadSpaceGroup to give lattice and crystal systems
%
%     Version 010 02-05-2012 by WLudwig
%       Add multi-phases materials
%
%     Version 009 08-03-2012 by PReischig
%       Updated to parameters version 2:
%         Modified fields: match, cryst, xop, acq, seg, rec
%         Added field    : labgeo, samgeo, version, fsim
%         Removed fields : geo, opt, match_calib
%
%     Version 008 14-12-2011 by LNervo
%       Clean code: y/n -> [y/n]
%                   ';' at the end of lines
%                   '()' at the end of functions/methods
%                   add 'clear' statements to free memory
%
%     Version 007 02-03-2012 by LNervo
%       Split crystallographic part into functions
%
%     Version 006 06-01-2012 by LNervo
%       Add fed structure
%
%     Version 005 01-12-2011 by LNervo
%       Adapt to new parameters file template
%       Cryst1 is made of cryst1, xop1, reflections1
%
%     Version 004 22-11-2011 by LNervo
%       Split acquisition structure 'acq' into acq, cryst, xop, geo, opt
%       Add version option for saving mat files
%
%     Version 003 17-11-2011 by LNervo
%       Change 7_FED into 7_fed
%       Add OAR_log folder
%       Add geo structure with detector geometry parameters
%       Use the same folder naming for the source code (see initialize_gt2.m)
%
%     Version 002 08-11-2011 by LNervo
%       *** Cleanest version (first attempt) ***
%       Add 3_pairmatching, 6_rendering, 7_FED and 8_optimization folders
%           in data analysis directory
%       Change 'true' 'false' in 1 - 0 logical values
%       Add arbitrary geometry parameters to acq structure
%       Add crystallography stuff to acq structure
%       Add xop to acq (create folder in /data/id11/graintracking/file_xop)
%
%     Version 001 06-04-2011 by AKing
%
%     FUNCTIONS COMPILED:
%[bin]- gtCopyCorrectUndistortCondor

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% title!
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp(' ');
disp(' ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~');
disp(' ~      Welcome to Grain Tracking at the ESRF!      ~');
disp(' ~                                                  ~');
disp(' ~     ...using the new parameters template...      ~');
disp(' ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~');
pause(1);
close('all');

if (isempty(whos('global', 'GT_MATLAB_HOME')))
    gtError('SETUP:no_such_variable', ...
          ['GT_MATLAB_HOME variable doesn''t exist in global workspace, '...
           'your environment is not sane, and you need to either ' ...
           're-initialise or re-run matlab.'])
end
global GT_MATLAB_HOME;

parameters = [];
list       = [];

% Initialise output colors for errors and warnings
defaultColor = sprintf(gtGetANSIColour());
redColor = sprintf(gtGetANSIColour('red'));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% check for a pre-existing parameters file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if exist(fullfile(pwd,'parameters.mat'), 'file')
    disp('A parameters.mat file already exists in this directory');
    disp('You can recreate this parameters file to enter new parameters');
    disp('or use the existing file and just start the correct/undistortion process straightaway');
    load('parameters.mat');

    check = inputwdefault('Launch copying without changing parameters? [y/n]', 'n');
    if strcmpi(check, 'y')
        disp(' ');
        % gtCopyCorrectUndistortWrapper handles condoring of copying
        disp(' starting data copying using condor');
        disp(' ');
        gtCopyCorrectUndistortWrapper(parameters);
        return;
    end
    check = inputwdefault('Do you want to reset all the values in parameters.mat? [y/n]', 'n');
    % build template locally from build_list_v# # is the version number
    % get also list which contains info of parameters fields.
    [tmppar, list] = make_parameters(2);
    if strcmpi(check, 'y')
        parameters = tmppar;
    end
    clear check tmppar;
else
    [parameters, list] = make_parameters(2);
end

if isempty(parameters.acq.dir)
    parameters.acq.dir = pwd;
end
list_name = fullfile(parameters.acq.dir, 'list.mat');
save(list_name, 'list');

% Add to xml file fields like 'collection_dir' and updates 'scanName' by
% reading some info from headers of ref0000_0000.edf  and darkend0000.edf to
% update xml file.
[xml_newname, tmpxml] = gtUpdateXmlFile();
if isempty(xml_newname)
    parameters.acq.name               = gtGetLastDirName(pwd());
    parameters.acq.collection_dir_old = pwd();
    parameters.acq.collection_dir     = pwd();
else
    parameters.acq.name               = tmpxml.acquisition.scanName;
    parameters.acq.collection_dir_old = tmpxml.acquisition.collection_dir_old;
    parameters.acq.collection_dir     = pwd();
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% directory for analysis - try to make sensible defaults
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% if in a visitor directory, make a DCT_Analysis directory.  Otherwise, use
% ID11/graintracking
parameters.acq.dir = gtSetAnalysisDir(pwd());

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Now check with user
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

hasChosen = false;
while (~hasChosen)
    dir = inputwdefault('Which directory do you want to use as analysis directory?', parameters.acq.dir);
    jdir = java.io.File(dir);
    if (~jdir.isAbsolute())
        disp([redColor 'Analysis directory should be an absolute path! Try again...' defaultColor]);
    else
        jdir.mkdirs();
        if (jdir.isFile())
            disp([redColor 'This already exists as a file. Choose another one...' defaultColor]);
        elseif (~jdir.canWrite())
            disp([redColor 'You have no writing permission to this directory! Choose another one...' defaultColor]);
        else
            parameters.acq.dir = dir;
            hasChosen = true;
        end
    end
end
freeSpace = round(jdir.getUsableSpace() / 10^6);
fprintf('You have about %d Mb free space\n', freeSpace);
jdir.delete();

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Set up the analysis directory
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% check the default names etc before making directories
% set up directory structure in this location

disp(['Setting up directory structure in ' parameters.acq.dir]);
[~,msg]=mkdir(parameters.acq.dir);                                               disp(msg);
[~,msg]=mkdir( fullfile(parameters.acq.dir, '0_rawdata') );                      disp(msg);
[~,msg]=mkdir( fullfile(parameters.acq.dir, '0_rawdata', parameters.acq.name) ); disp(msg);
[~,msg]=mkdir( fullfile(parameters.acq.dir, '0_rawdata', 'Orig') );              disp(msg);
[~,msg]=mkdir( fullfile(parameters.acq.dir, '1_preprocessing') );                disp(msg);
[~,msg]=mkdir( fullfile(parameters.acq.dir, '1_preprocessing', 'full') );        disp(msg);
[~,msg]=mkdir( fullfile(parameters.acq.dir, '1_preprocessing', 'abs') );         disp(msg);
[~,msg]=mkdir( fullfile(parameters.acq.dir, '1_preprocessing', 'ext') );         disp(msg);
[~,msg]=mkdir( fullfile(parameters.acq.dir, '2_difspot') );                      disp(msg);
[~,msg]=mkdir( fullfile(parameters.acq.dir, '3_pairmatching') );                 disp(msg);
[~,msg]=mkdir( fullfile(parameters.acq.dir, '4_grains') );                       disp(msg);
[~,msg]=mkdir( fullfile(parameters.acq.dir, '5_reconstruction') );               disp(msg);
[~,msg]=mkdir( fullfile(parameters.acq.dir, '6_rendering') );                    disp(msg);
[~,msg]=mkdir( fullfile(parameters.acq.dir, '7_fed') );                          disp(msg);
[~,msg]=mkdir( fullfile(parameters.acq.dir, '8_optimization') );                 disp(msg);
[~,msg]=mkdir( fullfile(parameters.acq.dir, 'OAR_log') );                        disp(msg);

parameters_name = fullfile(parameters.acq.dir, 'parameters.mat');
if (exist(parameters_name, 'file'))
    % if a parameters.mat already exists in the analysis directory, use it.
    p = load(parameters_name, 'parameters');
    p.parameters.acq.dir = parameters.acq.dir;
    parameters = p.parameters;
    disp('Using parameters file in the analysis directory...');
    disp(parameters_name);
    clear p;
else
    % Otherwise just load stuff from XML and guess default parameters

    % Parameters from .xml if possible
    xmlfname = fullfile(parameters.acq.collection_dir, [parameters.acq.name '.xml']);

    % try to read the .xml file that may be present
    try
        interactive = true;
        params_xml = gtLoadAcquisitionXML(xmlfname, interactive);
    catch mexc
        if gtCheckExceptionType(mexc, 'XML:no_xml_file', @()disp('Quitting..'))
            return;
        end
    end
    % update energy field from updated xml file
    if ~isempty(tmpxml)
        params_xml.energy = tmpxml.acquisition.energy;
        parameters.acq = gtAddMatFile(parameters.acq, params_xml, true, true, false);
    end
    clear params_xml;

    %%% And now let's initialise automatically everything we can.

    % Default ACQUISITION parameters
    params_acq = gtAcqDefaultParameters(parameters.acq);
    parameters.acq = gtAddMatFile(parameters.acq, params_acq, true, true, false);
    clear params_acq;

    % Default SAMPLE REFERENCE
    % Default voxel size is set to 1 lab unit, so samgeo is like labgeo at
    % omega=0.
    parameters.samgeo = gtGeoSamDefaultParameters();
end

% now get or confirm the important values:
header = 'Check carefully that the following are correct:';
    % some of this is always from xml
sublist = gtSubList(list.acq, 'collection_dir', 'name', 'dir', ...
    'xdet', 'ydet', 'nproj', 'pixelsize', 'count_time', 'energy', ...
    'dist', 'sensortype', 'type', 'interlaced_turns', 'rotation_axis', ...
    'distortion', 'no_direct_beam', 'detector_definition', 'flip_images', ...
    'online');

parameters.acq = gtModifyStructure(parameters.acq, sublist, header);

% based on this, some extra info might be needed
if strcmpi(parameters.acq.rotation_axis, 'horizontal')
     sublist = gtSubList(list.acq, 'rotation_direction');
     parameters.acq = gtModifyStructure(parameters.acq, sublist, ...
         'Images will be rotated for processing:');
end

disp('Saving parameters.mat in the analysis directory...');
save(parameters_name, 'parameters');
clear xmlfname;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Geometric parameters of setup in Lab reference
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% get default values for labgeo
parameters.labgeo = gtGeoLabDefaultParameters(parameters.acq);

% 2theta angle range seen by the detector
R       = parameters.acq.maxradius * parameters.acq.pixelsize;
detnorm = cross(parameters.labgeo.detdiru, ...
                parameters.labgeo.detdirv);
[minimum, maximum] = gtGeoLimitsTwoTheta(R, parameters.labgeo.detrefpos, detnorm);
parameters.labgeo.detanglemin = minimum;
parameters.labgeo.detanglemax = maximum;

clear minimum maximum R;

header = 'Parameters dealing with arbitrary geometry:';
% note other fields are calculated, typically during preprocessing when we
% have the direct beam bounding box.
sublist = gtSubList(list.labgeo, 'beamdir', 'rotdir', 'rotpos', 'detrefpos', ...
    'detrefu', 'detrefv', 'detdiru', 'detdirv', 'pixelsizeu', 'pixelsizev', ...
    'labunit', 'deflabX', 'deflabY', 'deflabZ');
parameters.labgeo = gtModifyStructure(parameters.labgeo, sublist, header);

%header = 'Parameters dealing with arbitrary geometry:';
%parameters.samgeo = gtModifyStructure(parameters.samgeo, list.samgeo(:,1:2), header);

disp('Saving parameters.mat in the analysis directory...');
save(parameters_name, 'parameters');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Crystallography setup -  allow for multiphase materials
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nof_phases = 1;
parameters.acq.nof_phases = inputwdefaultnumeric( ...
    'Number of crystallographic phases to be analysed in this sample?', num2str(nof_phases));

% get Cryst field information for all phases
for phase_id = 1 : parameters.acq.nof_phases
    % make a subfolder in 4_grains
    [~, msg] = mkdir( fullfile(parameters.acq.dir, '4_grains', ...
        sprintf('phase_%02d', phase_id)) );
    disp(msg);
    parameters = gtFillCrystFields(parameters, phase_id);
end % end crystallography

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Set up the database tables
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
gtDBConnect();
try
    disp(['setting up database tables for ' parameters.acq.name]);
    gtDBCreateDifblobTable(parameters.acq.name);
    gtDBCreateDifspotTable(parameters.acq.name);
    gtDBCreateExtspotTable(parameters.acq.name);
    if strcmpi(parameters.acq.type, '360degree')
        gtDBCreateSpotPairTable(parameters.acq.name, 0, 0);
    end
catch mexc
    gtPrintException(mexc, 'failed to set up tables! try again, forcing overwite of existing tables?')
    check=inputwdefault('Overwrite tables? [y/n]', 'n');
    if strcmpi(check,'y')
        gtDBCreateDifblobTable(parameters.acq.name, 1);
        gtDBCreateDifspotTable(parameters.acq.name, 1);
        gtDBCreateExtspotTable(parameters.acq.name, 1);
        if strcmpi(parameters.acq.type, '360degree')
            gtDBCreateSpotPairTable(parameters.acq.name, 1, 0);
        end
    end
end
clear check;

disp('Saving parameters.mat in the analysis directory...');
save(parameters_name,'parameters');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PREP - for preprocessing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (parameters.acq.no_direct_beam)
    % if no direct beam, don't normalise images
    parameters.prep.normalisation = 'none';
else
    parameters.prep.normalisation = 'margin';
end

% Andy 26/4/2011 - problem with a scan with 1440/720 images when using default
% values.
% Must check that fullint and absint values work.  n x fullint and m x absint
% should equal total number of images.   I think it should also equal the
% number of images per turn for an interlaced scan.
if strcmp(parameters.acq.type, '360degree')
    % images per turn in an interlaced scan
    n = 2*parameters.acq.nproj/(parameters.acq.interlaced_turns+1);
elseif strcmp(parameters.acq.type, '180degree')
    n = parameters.acq.nproj;
end
test = mod(n, 1:100);
test = find(test == 0); % which numbers work?
% get the working numbers closest to 10 and 50 respectively
test10 = abs(test-10); % difference from 10
[~, ind10] = min(test10);
parameters.prep.absint     = test(ind10);
parameters.prep.absrange   = 10*parameters.prep.absint;
test50 = abs(test-50); % difference from 50
[~, ind50] = min(test50);
parameters.prep.fullint    = test(ind50);
parameters.prep.fullrange  = 10*parameters.prep.fullint;
parameters.prep.margin     = 5;
parameters.prep.intensity  = 1800;
parameters.prep.filtsize   = [3 3];
parameters.prep.drifts_pad = 'av';
%for interlaced scans
if parameters.acq.interlaced_turns > 0
    parameters.prep.renumbered = false;
end
parameters.prep.bbox          = [];
parameters.prep.correct_drift = '';
parameters.prep.udrift        = [];
parameters.prep.udriftabs     = [];
parameters.prep.vdrift        = [];

disp('Saving parameters.mat in the analysis directory...');
save(parameters_name, 'parameters');

clear test n test10 test50 dum ind10 ind50;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SEG - for segmentation of diffraction blobs / spots
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parameters.seg = gtSegDefaultParameters();

disp('Saving parameters.mat in the analysis directory...');
save(parameters_name, 'parameters');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MATCH - for Friedel pair matching
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parameters.match = gtMatchDefaultParametersGUI();
parameters.match.thetalimits = [parameters.labgeo.detanglemin, ...
                                parameters.labgeo.detanglemax];

disp('Saving parameters.mat in the analysis directory...');
save(parameters_name, 'parameters');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% INDEX - used by the indexing algorithm INDEXTER
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parameters.index = gtIndexDefaultParameters();

disp('Saving parameters.mat in the analysis directory...');
save(parameters_name, 'parameters');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FORWARD SIMULATION PARAMETERS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parameters.fsim = gtFsimDefaultParameters();
disp('Saving parameters.mat in the analysis directory...');
save(parameters_name, 'parameters');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% RECONSTRUCTION - parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parameters.rec = gtRecDefaultParameters();
disp('Saving parameters.mat in the analysis directory...');
save(parameters_name, 'parameters');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FED part
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parameters.fed.dir           = parameters.acq.dir;
parameters.fed.dct_vol       = fullfile(parameters.acq.dir, 'dct_vol.edf');
parameters.fed.dct_offset    = [0 0 0];
parameters.fed.renumber_list = [];

disp('Saving parameters.mat in the analysis directory...');
save(parameters_name, 'parameters');
disp(' ');
close('all');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% recompile binaries - not often needed.  But more often than it should be...
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
check = inputwdefault('Recompile copy function for OAR? Not normally needed! [y/n]', 'n');
if strcmpi(check, 'y')
    disp('recompiling gtCopyCorrectUndistortCondor');
    gt_mcc('gtCopyCorrectUndistortCondor');
end

disp(' ');
disp('Changing to the analysis directory');
disp(' ');
cd(parameters.acq.dir);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% start copy/move/undistortion process
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
check = inputwdefault('Start copy/move/undistortion process now? [y/n]', 'y');
if strcmpi(check, 'n')
    disp('You can start the process later by running:');
    disp('    load(parameters)');
    disp('    gtCopyCorrectUndistortWrapper(parameters)');
    return;
end

% gtCopyCorrectUndistortWrapper handles oaring of copying
disp('Starting data copying using OAR');
disp(' ');
gtCopyCorrectUndistortWrapper(parameters);

clear msg check;

end % end of function
