function gtShowFsim(grain, phaseid, clims)
% GTSHOWFSIM  Shows the forward simulation result for one grain
%   Usage:
%       gtShowFsim(grainid, phaseid, clims)
%       gtShowFsim(grain{grainid}, phaseid, clims)
%           (in this case, phase ID is ignored)

    if isnumeric(grain)
        if ~exist('phaseid', 'var') || isempty(phaseid)
            phaseid = 1;
        end
        filename = fullfile('4_grains',sprintf('phase_%02d',phaseid),sprintf('grain_%04d.mat', grain));
        grain = load(filename);
    end
    if (~exist('clims', 'var'))
        clims = 0;
    end

    grainid = grain.id;
    phaseid = grain.phaseid;
    
    parameters = [];
    load('parameters.mat');
    datasetname = parameters.cryst(phaseid).name;
    % connect to database
    gtDBConnect();

    % look for dispots in conflicts
    hasConflictInfo = exist(fullfile('4_grains','grains_conflicts.mat'), 'file');
    if (hasConflictInfo)
        grains_conflicts = [];
        load(fullfile('4_grains','grains_conflicts.mat'));

        idspots = grains_conflicts{phaseid}.getIdSpotRow(grainid,:);

        if ~isempty(idspots)
            fprintf('Found %d spots in common with other grain(s)\n',numel(idspots))
            disp(idspots)
            
            numSpots = numel(idspots);
            id = zeros(numSpots, 1);
            x  = zeros(numSpots, 1);
            y  = zeros(numSpots, 1);
            im = zeros(numSpots, 1);

            for ii = 1:numSpots
                query = sprintf(...
                    [ 'SELECT difspotID, CentroidX, CentroidY, CentroidImage ' ...
                      '     FROM %sdifspot' ...
                      '     WHERE difspotID = %d' ], ...
                    datasetname, idspots(ii));
                [id(ii), x(ii), y(ii), im(ii)] = mym(query);
            end

            idconflicts = grains_conflicts{phaseid}.getIdConflict(grainid);
            for kk = 1:numel(idconflicts)
                [~, indexes] = intersect(grain.difspotID, idspots, 'stable');
                indexes = indexes';
                fprintf('  idconflict : %d   indexes: ', idconflicts(kk))
                fprintf('%d \n', indexes)
                flags = grain.flag';
                selected = find(flags >= parameters.fsim.selected);
                disp('  Original flags for difspots:')
                disp(flags(selected(indexes)))
                flags(selected(indexes)) = 6;
                grain.oldflag = grain.flag;
                grain.flag = flags;
                grain.conflicts = selected(indexes);
            end
        else
            fprintf('No difspots in conflict found for grain %d\n', grainid)
        end % end if ~isempty(idspots)
        
    else
        disp('The file grains_conflicts.mat has not been found.')
        disp('First create it running gtAnalyseGrainsConflicts and gtSolveGrainsConflicts.')
    end % end hasConflictInfo

    clear grains_conflicts  
    
    leg = { ...
        '1: No intensity in raw images', ...
        '2: Intensity in raw images', ...
        sprintf('3: Segmented intensity not\n    matching search criteria'), ...
        '4: Difspot matching search criteria', ...
        '5: Indexed difspot as Friedel pair', ...
        '6: Spot claimed by another grain'};
   
    markers = {'rx','mo','bo','gd','gh','yo'};
    
    % take uvw positions and omega values for grain
    uvw = grain.allblobs.uvw;
    omega = grain.allblobs.omega;
    
    % get indexes of grains on detector
    ondet = grain.ondet;
    
    % create figure
    f = figure('Units','normalized','Position',[0.1 0.1 0.75 0.8]);

    % create axes
    ax = axes('Parent', f, 'Units','normalized','Position', [0.1 0.1 0.775 0.8],'Tag','flag');
    colormap(ax, gray);
    freezeColors(ax)
    hold(ax, 'on')
    
    title(sprintf('Phase name "%s"     Phase number %02d     GrainID %d', strrep(datasetname,'_','\_'), phaseid, grainid));

    % change color limits
    clims = gtSetColourLimits(grain.full, clims);

    imagesc(grain.full, 'Parent', ax, clims);
    axis(ax, 'equal');
    axis(ax, [1 size(grain.full, 2) 1 size(grain.full, 1)])
    gtBrightnessContrast(ax);
    
    % flag = 0  no spot search - just show expected positions
    ind = find(grain.flag == 0);
    if ~isempty(ind == length(grain.flag))
        u_vec = uvw(ondet(ind), 1);
        v_vec = uvw(ondet(ind), 2);
        w_vec = uvw(ondet(ind), 3);
        om_vec = omega(ondet(ind));
        leg = 'expected spot positions';
    end

    h = [];
    ind = [];
    for ii=1:6
        ind = find(grain.flag == ii);
        if ~isempty(ind)
            u_vec{ii}  = uvw(ondet(ind),1);
            v_vec{ii}  = uvw(ondet(ind),2);
            w_vec{ii}  = uvw(ondet(ind),3);
            om_vec{ii} = omega(ondet(ind));
        else
            u_vec{ii}  = 1;
            v_vec{ii}  = -10;
            w_vec{ii}  = [];
            om_vec{ii} = [];
        end
    end
    
    if ~isempty(idspots)
        set(h(6), 'MarkerSize', 20);
        % Only in case we have it
        text(x,y,num2str(id'),'Color','y')
    end
           
    uv_ax = copyobj(ax, gcf);
    set(uv_ax, 'Tag', 'omega')
    delete(findobj(uv_ax,'Type','image'))
    set(uv_ax, 'Visible', 'off');
    
    for ii=1:length(u_vec)
        if ~isempty(u_vec{ii})
            h(ii) = scatter(uv_ax, u_vec{ii}, v_vec{ii}, 20, markers{ii});
        end
    end
    
    legend(h, leg, 'Position', [0.8137 0.8110 0.1738 0.1384], 'Parent', f);    

    colormap(ax, gray)
    
    h_u = uicontrol( ...
        'Parent', f, ...
        'Style', 'popup', ...
        'Units','normalized', ...
        'String', ' flag colored | omega colored ', ...
        'Position', [0.8237 0.6 0.15 0.1], ...
        'Value', 1, ...
        'Callback', {@choosemap, markers, u_vec, v_vec, om_vec, [ax uv_ax], leg});

end % end of function

%% sub-functions
%

function choosemap(hObj,event,markers,u,v,om,ha,leg) %#ok<INUSD>
    % Called when user activates popup menu 
    val = get(hObj,'Value');
    
    uv_ax = ha(2);
    im_ax = ha(1);
    
    if val == 1
        disp('flag coloring')
 
        colormap(im_ax, gray)
        
        delete(findobj(gcf, 'type', 'hggroup'));
        delete(findobj(gcf, 'tag', 'Colorbar'))
        delete(findobj(gcf, 'tag', 'legend'))
        
        h = [];
        for ii=1:length(u)
            if ~isempty(u{ii})
                h(ii) = scatter(uv_ax, u{ii}, v{ii}, 20, markers{ii});
            end
        end

        assignin('caller','h',h);
        
        legend(h, leg, 'Position', [0.8137 0.8110 0.1738 0.1384], 'Parent', gcf);
       
    elseif val == 2
        disp('omega coloring')
                         
        cmap = jet(1001);
        omtoRGB = @(om) cmap(round((om)/(360)*1000+1),:);
        colormap(uv_ax, cmap)
        
        hc = colorbar('peer', uv_ax);
        set(hc, 'Position', [0.77 0.1 0.0161 0.8])
        caxis(uv_ax, [0 360])
        cbfreeze(hc)
        
        delete(findobj(gcf, 'type', 'legend'));
        delete(findobj(gcf, 'type', 'hggroup'));

        legend(uv_ax, 'hide')
        legend(im_ax, 'hide')

        
        for ii=1:length(u)
            if ~isempty(om{ii})
                h(ii) = scatter(uv_ax, u{ii}, v{ii}, 20, omtoRGB(om{ii}), 'filled');
            end
        end
        
        assignin('caller','h',h);
        
        colormap(im_ax, gray)
        
    elseif val == 3
        disp('intensity coloring')
      
    end    

end
