function [f_handle, p_handle] = gtDrawGrainUnitCells(grains, varargin)
%     [f_handle, p_handle] = gtDrawGrainUnitCells(grains, varargin)
%     -------------------------------------------------------------
%     Generalized cases to plot more than one dataset.
%
%     Draws a unit cell representing grain size, location and orientation by  
%     cubes or hexagonal prismes based on the their Rodrigues vectors.
%
%     INPUT:
%       grains      = <cell array>   grain cell-structure from indexing
%
%     OPTIONAL INPUT (varargin):
%       ids         = <double>       list of grain IDs of interest; by default all
%                                    the grains are drawn
%       cmap        = <double>       can be a component of the strain tensor
%                                    {grain.strain.strainT(3,3)} or a colour map
%       hlight      = <double>       grain id-s to be highlighted {[]}
%       linecolor   = <string>       color for edges
%       strain      = <double>       scale factor for the strain values, if wanted {0}
%                                    (zero to use the unstrained unit cell)
%       alpha       = <double>       transparency value for patches {1}
%       pxsize      = <double>       pixel size (mm/px); if empty, taken from
%                                    parameters.labgeo
%       ratio       = <double>       c/a ratio; if empty, taken from
%                                    parameters.cryst
%       phaseid     = <int>          phase number {1}
%       type        = <string>       unit cell type; if empty, taken from
%                                    parameters.cryst
%       label       = <string>       name for entry in the legend <string>
%
%       hf          =                figure handle if already existing {[]}
%       zoom        = <double>       zoom in of this quantity {1.2}
%       view        = <double>       3D angular view {[0 90]} (1x2)
%       caxis       = <logical>      flag to draw the c-axis {false}
%       legend      = <logical>      display or not the legend {false}
%       legendpos   = <double>       location of the legend box {'NorthEast'}
%
%     OUTPUT:
%       f_handle    =                figure handle
%       p_handle    =                patches handle
%
%
%     Usage:
%        gtDrawGrainUnitCells({grain}, 'ids', {1:10}, 'cmap', {gtRandCmap(10)}, 'pxsize', {0.0014})   
%
%
%     Version 006 06-06-2013 by LNervo
%       Added function to compute unit cell vertices, modified
%       gtPlotGrainUnitCell
%
%     Version 005 11-03-2013 by LNervo
%       Use cell arrays to plot multiple datasets
%
%     Version 004 15-11-2012 by LNervo
%       Use gtDrawSampleGeometry(labgeo) to create the graphics with axis and reference
%       system
%
%     Version 003 23-10-2012 by LNervo
%       Add varargin


app.ids       = arrayfun(@(num) 1:numel(grains{num}), 1:numel(grains), 'UniformOutput', false);
app.cmap{1}   = [];
app.hlight{1} = [];
app.linecolor = {[0 0 0],[1 0 0],[0 1 0],[0 0 1],[1 1 0],[1 0 1],[0 1 1],[1 1 1]};
app.strain    = cellfun(@(num) 0, grains, 'UniformOutput', false);
app.alpha     = cellfun(@(num) 1, grains, 'UniformOutput', false);
app.pxsize{1} = [];
app.ratio{1}  = [];
app.phaseid   = cellfun(@(num) 1, grains, 'UniformOutput', false);
app.type{1}   = [];
app.label{1}  = [];

app.hf        = [];
app.zoom      = 1.2;
app.view      = [0 90];
app.caxis     = false;
app.legend    = false;
app.legendpos = 'NorthEast';

app = parse_pv_pairs(app, varargin);

app.linecolor = app.linecolor(1:numel(grains));

parameters = [];
load('parameters.mat');
labgeo  = parameters.labgeo;

if isempty(app.pxsize{1})
    app.pxsize{1}  = (labgeo.pixelsizeu + labgeo.pixelsizev)/2;
end
if isempty(app.hf)
    hf = gtDrawSampleGeometry(labgeo.samenvrad, labgeo.samenvbot, labgeo.samenvtop);
else
    hf = app.hf;
end

set(hf, 'Visible', 'off');

hp = [];

% loop over datasets
for ii=1:numel(grains)

    if ii>1
        if numel(app.type)==1,   app.type{ii}   = app.type{1}; end
        if numel(app.ratio)==1,  app.ratio{ii}  = app.ratio{1}; end
        if numel(app.pxsize)==1, app.pxsize{ii} = app.pxsize{1}; end
        if numel(app.cmap)==1,   app.cmap{ii}   = app.cmap{1}; end
        if numel(app.hlight)==1, app.hlight{ii} = app.hlight{1}; end
        if numel(app.label)==1,  app.label{ii}  = app.label{1}; end
    end

    cryst = parameters.cryst(app.phaseid{ii});
    if isempty(app.type{ii}),  app.type{ii}  = cryst.crystal_system; end
    if isempty(app.ratio{ii}), app.ratio{ii} = cryst.latticepar(3)/cryst.latticepar(1); end
    if isempty(app.ids{ii}),   app.ids{ii}   = 1:length(grains{ii}); end
    if isempty(app.cmap{ii}),  app.cmap{ii}  = repmat([1 0 0],length(grains{ii}),3); end %red color
    
    ids   = app.ids{ii};
    grain = grains{ii};
    cmap  = app.cmap{ii};
    
    % if single value (RGB), copy it for all the grains
    if size(cmap,1)==1 && size(cmap,2)>1
        cmap = repmat(cmap,length(grain),1);
    end
    % remove bkg color
    if all(cmap(1,:) == [0 0 0])
        cmap = cmap(2:end,:);
    end
        
    % get selected grains for current datasets
    grain = grain(ids);
    cmap  = cmap(ids,:);
    
    % vertices and faces calculation : TO DO LIST : unify in one function
    if strcmpi(app.type{ii}, 'cubic')
      data{ii} = gtCubicUnitCell('ratio',app.ratio{ii},'draw',false,'centered',true);
    elseif strcmpi(app.type{ii}, 'hexagonal')
      data{ii} = gtHexagonalUnitCell('ratio',app.ratio{ii},'draw',false,'centered',true);
    end
    
    % draw unit cells
    hold(gca,'on');

    p = [];
    for jj = 1:length(grain)

        % grain size
        if isfield(grain{jj}, 'stat') 
            if isfield(grain{jj}.stat, 'size_int')
                grain{jj}.radius = 0.5*app.pxsize{ii}*grain{jj}.stat.size_int; % mm
            elseif isfield(grain{jj}.stat, 'bbxsmean') && isfield(grain{jj}.stat, 'bbysmean')
                grain{jj}.radius = 0.5*app.pxsize{ii}*(grain{jj}.stat.bbxsmean + grain{jj}.stat.bbysmean)/2; % mm
            end
        else
            grain{jj}.radius = 10;
        end
        % grain strain
        if ~isfield(grain{jj}, 'strain')
            grain{jj}.strain.strainT = NaN(3);
        end

        % highlight grain?
        if ismember(jj, app.hlight{ii}), hl = true; else hl = false; end

        % computes vertices and caxis coordinates
        vertices{jj} = gtComputeGrainUnitCell(grain{jj}.R_vector, data{ii}.vertices, ...
            grain{jj}.radius, grain{jj}.center, grain{jj}.strain.strainT, app.strain{ii});
        if app.caxis
            caxis{jj} = gtComputeGrainUnitCell(grain{jj}.R_vector, data{ii}.caxis, ...
                grain{jj}.radius, grain{jj}.center, grain{jj}.strain.strainT, app.strain{ii});
        else
            caxis{jj} = [];
        end
        % draws patches for prismatic planes
        p{end+1} = gtPlotGrainUnitCell(vertices{jj}, caxis{jj}, data{ii}.faces, ...
            cmap(jj,:), hl, app.linecolor{ii}, app.alpha{ii});
          
        set(p{end}, 'UserData', [ii ids(jj)]);
        set(p{end}, 'Tag', sprintf('data_%02d_grain_%04d',ii,grain{jj}.id));   
        
        userdata(jj).grain = struct('id', grain{jj}.id, 'center', grain{jj}.center, 'R_vector', grain{jj}.R_vector, ...
                                    'strainT', grain{jj}.strain.strainT, 'strain', app.strain{ii}, ...
                                    'radius', grain{jj}.radius, 'cmap', cmap(jj,:), ...
                                    'caxis', caxis{jj}, 'vertices', vertices{jj}, 'dataset', ii);
        
    end % end for jj % loop over grains

    app.cmap{ii} = cmap;
    
    plotdata{ii} = userdata;
    hp{ii} = p;

end % end for ii % loop over datasets

app.label = app.type;

axis equal;
zoom('out');
zoom(app.zoom)
zoom('off')
view(app.view);

if app.legend
    leg = legend('Parent',hf,'String',app.label,'Location',app.legendpos);
end

set(hf, 'UserData', plotdata);
set(hf, 'Visible', 'on');
f_handle = hf;
app.hf = f_handle;

if nargout == 2
    p_handle = hp;
    app.hp = hp;
end

print_structure(app, 'general settings:')

end % end of function
