function [Bmat, Amat] = gtCrystHKL2CartesianMatrix(lp)
%
%     [Bmat, Amat] = gtCrystHKL2CartesianMatrix(lp)
%     ---------------------------------------------
%     Returns a 3x3 matrix that can be used for coordinate transform from 
%     a signed HKL into a Cartesian vector in real space and for any lattice 
%     and unit cell.
%
%     The Cartesian (orthogonal) basis (X,Y,Z) is defined such that:
%       - lattice vectors of the unit cell are a1, a2, a3 (not orthogonal)
%       - X is parallel with a1
%       - Y lies in the a1-a2 plane
%       - Z is cross(X,Y)
% 
%     The definition corresponds to the one in Poulsen's 3DRXRD book, Chapter
%     3. This function essentially computes the B matrix in the equation 
%     Gc = B*Ghkl  (used with column vectors!)
%
%     INPUT:
%       lp = unit cell lattice parameters in real space 
%            [a b c alpha beta gamma] (angles are in degrees)
%
%     OUTPUT:
%       Bmat = the transformation matrix for column vectors (!)
%       Amat = the A matrix too (real space) for dealing with UVW 
%              lattice directions.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Lattice vectors in real space, in a Cartesian basis
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Calculate 'a1' and 'a2' lattice vector X,Y,Z coordinates:
a1 = [lp(1)             0                 0];
a2 = [lp(2)*cosd(lp(6)) lp(2)*sind(lp(6)) 0];

% Calculate 'a3' components in X-Y plane:
x3 = lp(3)*cosd(lp(5));
y3 = lp(3)*(cosd(lp(4))-cosd(lp(5))*cosd(lp(6)))/sind(lp(6));

% Calculate 'a3' Z coordinate from length
z3 = sqrt(lp(3)^2 - x3^2 - y3^2);

% Assemble 'a3'
a3 = [x3 y3 z3];


% Volume of spanned by 'a'-s (determinant of matrix of a-s)
crossa2a3 = [a2(2)*a3(3)-a2(3)*a3(2); a2(3)*a3(1)-a2(1)*a3(3); a2(1)*a3(2)-a2(2)*a3(1)];
avol = a1*crossa2a3;

% for output
Amat=[a1' a2' a3'];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Reciprocal lattice vectors in the Cartesian basis
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Reciprocal lattice vectors by definition:
%  b1 = cross(a2,a3)/avol
%  b2 = cross(a3,a1)/avol
%  b3 = cross(a1,a2)/avol

b1 = crossa2a3/avol;
b2 = [a3(2)*a1(3)-a3(3)*a1(2); a3(3)*a1(1)-a3(1)*a1(3); a3(1)*a1(2)-a3(2)*a1(1)]/avol;
b3 = [a1(2)*a2(3)-a1(3)*a2(2); a1(3)*a2(1)-a1(1)*a2(3); a1(1)*a2(2)-a1(2)*a2(1)]/avol;

Bmat = [b1 b2 b3];


end % of function
