/**
 * Adds a grain to the volume, taking care of the superposition between grains.
 * The shared voxels are put to -1, while the others are set to the index of the
 * grain.
 *
 * Nicola Vigano', 2012, ID11 @ ESRF vigano@esrf.eu
 */

#include "mex.h"

#include "GtMatlabVolumeLooper.h"

template<typename TypeIn>
void extDoAssignment(mxArray * output, void * vol, const double * limits,
                  const mwSize * dimsOut, const mwSize * dimsVol,
                  const double & index);

template<typename TypeOut, typename TypeIn>
class GtMatlabVolumeInterference : public GtMatlabVolumeLooper<TypeOut, TypeIn> {
protected:
  void doWork(TypeOut & dest, const TypeIn & orig)
  {
    if (orig) { dest = (TypeOut) (-1 + (! dest) * this->index); }
  }
public:
  GtMatlabVolumeInterference(void * _out, void * _vol, const double * limits,
      const mwSize * dims_out, const mwSize * dims_vol, const double & indx)
  : GtMatlabVolumeLooper<TypeOut, TypeIn>(_out, _vol, limits, dims_out, dims_vol, indx)
  { }
};

template<typename TypeOut, typename TypeIn>
class GtMatlabVolumeInterferenceValue : public GtMatlabVolumeLooper<TypeOut, TypeIn> {
protected:
  void doWork(TypeOut & dest, const TypeIn & orig)
  {
    if (orig) { dest = (TypeOut) (-1 + (! dest) * orig); }
  }
public:
  GtMatlabVolumeInterferenceValue(void * _out, void * _vol, const double * limits,
      const mwSize * dims_out, const mwSize * dims_vol)
  : GtMatlabVolumeLooper<TypeOut, TypeIn>(_out, _vol, limits, dims_out, dims_vol, 0)
  { }
};

template<typename TypeOut, typename TypeIn>
inline void
assign(void * _out, void * _vol, const double * limits,
    const mwSize * dims_out, const mwSize * dims_vol, const double & indx)
{
  GtMatlabVolumeLooper<TypeOut, TypeIn> * looper = NULL;
  if (indx) {
    looper = new GtMatlabVolumeInterference<TypeOut, TypeIn>(_out, _vol, limits, dims_out, dims_vol, indx);
  } else {
    looper = new GtMatlabVolumeInterferenceValue<TypeOut, TypeIn>(_out, _vol, limits, dims_out, dims_vol);
  }
  looper->loopOver3D();
  delete looper;
}

void mexFunction( int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[] )
{
  /* Incoming image dimensions */
  const mwSize * dimsOut = mxGetDimensions(prhs[0]);
  const mwSize * dimsVol = mxGetDimensions(prhs[1]);

  /* Value of the grain */
  /* Used to optimize the computation (gcc is not saving this intermediate
   * result in a register, so this imposes it) */
  const double index = *mxGetPr(prhs[2]) +1;

  /* Limits of the grain bounding box */
  const double * const limits = mxGetPr(prhs[3]);

  /* -- No assert on correctness of parameters -- */

  /* Don't copy, just modify the incoming matrix */
  plhs[0] = (mxArray *) prhs[0];

  switch(mxGetClassID(prhs[1])) {
    case mxDOUBLE_CLASS: {
      extDoAssignment<const double>(plhs[0], mxGetData(prhs[1]), limits, dimsOut, dimsVol, index);
      break;
    }
    case mxSINGLE_CLASS: {
      extDoAssignment<const float>(plhs[0], mxGetData(prhs[1]), limits, dimsOut, dimsVol, index);
      break;
    }
    case mxINT8_CLASS: {
      extDoAssignment<const char>(plhs[0], mxGetData(prhs[1]), limits, dimsOut, dimsVol, index);
      break;
    }
    case mxUINT8_CLASS: {
      extDoAssignment<const unsigned char>(plhs[0], mxGetData(prhs[1]), limits, dimsOut, dimsVol, index);
      break;
    }
    case mxINT16_CLASS: {
      extDoAssignment<const short int>(plhs[0], mxGetData(prhs[1]), limits, dimsOut, dimsVol, index);
      break;
    }
    case mxUINT16_CLASS: {
      extDoAssignment<const unsigned short int>(plhs[0], mxGetData(prhs[1]), limits, dimsOut, dimsVol, index);
      break;
    }
    case mxINT32_CLASS: {
      extDoAssignment<const int>(plhs[0], mxGetData(prhs[1]), limits, dimsOut, dimsVol, index);
      break;
    }
    case mxUINT32_CLASS: {
      extDoAssignment<const unsigned int>(plhs[0], mxGetData(prhs[1]), limits, dimsOut, dimsVol, index);
      break;
    }
    case mxINT64_CLASS: {
      extDoAssignment<const long int>(plhs[0], mxGetData(prhs[1]), limits, dimsOut, dimsVol, index);
      break;
    }
    case mxUINT64_CLASS: {
      extDoAssignment<const unsigned long int>(plhs[0], mxGetData(prhs[1]), limits, dimsOut, dimsVol, index);
      break;
    }
    default:
      break;
  }
}

template<typename TypeIn>
void
extDoAssignment(mxArray * output, void * vol, const double * limits,
                  const mwSize * dimsOut, const mwSize * dimsVol,
                  const double & index)
{
  switch(mxGetClassID(output)) {
    case mxDOUBLE_CLASS: {
      assign<double, TypeIn>(mxGetData(output), vol, limits, dimsOut, dimsVol, index);
      break;
    }
    case mxSINGLE_CLASS: {
      assign<float, TypeIn>(mxGetData(output), vol, limits, dimsOut, dimsVol, index);
      break;
    }
    case mxINT8_CLASS: {
      assign<char, TypeIn>(mxGetData(output), vol, limits, dimsOut, dimsVol, index);
      break;
    }
    case mxUINT8_CLASS: {
      assign<unsigned char, TypeIn>(mxGetData(output), vol, limits, dimsOut, dimsVol, index);
      break;
    }
    case mxINT16_CLASS: {
      assign<short int, TypeIn>(mxGetData(output), vol, limits, dimsOut, dimsVol, index);
      break;
    }
    case mxUINT16_CLASS: {
      assign<unsigned short int, TypeIn>(mxGetData(output), vol, limits, dimsOut, dimsVol, index);
      break;
    }
    case mxINT32_CLASS: {
      assign<int, TypeIn>(mxGetData(output), vol, limits, dimsOut, dimsVol, index);
      break;
    }
    case mxUINT32_CLASS: {
      assign<unsigned int, TypeIn>(mxGetData(output), vol, limits, dimsOut, dimsVol, index);
      break;
    }
    case mxINT64_CLASS: {
      assign<long int, TypeIn>(mxGetData(output), vol, limits, dimsOut, dimsVol, index);
      break;
    }
    case mxUINT64_CLASS: {
      assign<unsigned long int, TypeIn>(mxGetData(output), vol, limits, dimsOut, dimsVol, index);
      break;
    }
    default:
      break;
  }
}
