classdef GtBaseGuiElem < handle
    properties
        conf;

        children_gui;
        ui_timers;
        ui_callbacks;

        timestamp = 0;

        cache = GtSimpleCache();
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Public object API - Interaction with outside
    methods (Access = public)
        function obj = GtBaseGuiElem()
        % GTBASEGUIELEM/GTBASEGUIELEM Constructor
            obj.conf = [];

            % Internal parameters
            obj.children_gui = {};
            obj.ui_timers = {};
            obj.ui_callbacks = {};
            % default parameters
            obj.conf.parent = [];
            obj.conf.f_title = '';
        end

        function delete(obj)
        % GTBASEGUIELEM/DELETE Destructor
            obj.guiQuit();
            clear obj.conf;
            clear obj;
        end

        function updateDisplay(obj)
        % GTBASEGUIELEM/UPDATEDISPLAY Displaying function
            if ((now - obj.timestamp) > 3.818655386567116e-07)
                obj.doUpdateDisplay();
                obj.timestamp = now;
            end
        end
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Internal methods - Addressable from derived classes
    methods (Access = protected)
        function initGtBaseGuiElem(obj, arguments)
        % Base function that initializes the GUI object and shows it.
        % To change or enhance the gui, please modify accordingly the methods
        % called by this.

            % Create the configuration
            obj.initParams(arguments);
            % Set parameters that shouldn't be changed by the user
            obj.setConfigInvariants();
            % Create the gui objects
            obj.initGui();

            % Once we have the structures registered into the guidata, we can set
            % the parameters corresponding to the volume
            obj.resetUiComponents();
            % Now we can add the callbacks
            obj.addUICallbacks();

            % Finally update the display, to visualize the objects
            obj.updateDisplay();

            % protect the window from being accidentall reused
            % Disabled because it prevents update
            %set(obj.conf.h_figure, 'handlevisibility', 'callback')
        end

        function initParams(obj, arguments)
            % parse input
            obj.conf = parse_pv_pairs(obj.conf, arguments);
        end

        function setConfigInvariants(obj)
            % Fixed parameters
            obj.conf.close_callback = @delete;
        end

        function initGui(obj)
        % If you overload it, please always call it as first!
            %disp(obj.conf.parent)
            if (isempty(obj.conf.parent))
                obj.conf.h_figure = figure( 'Name', obj.conf.f_title, ...
                                        'NumberTitle', 'off', 'MenuBar', 'none', ...
                                        'Toolbar', 'none');
                obj.addUICallback(obj.conf.h_figure, 'CloseRequestFcn', @(src, evt)delete(obj));
                obj.conf.currentParent = obj.conf.h_figure; % Where are incapsulating in

                % Resize
                screenPixels = GtBaseGuiElem.getPixels(0, 'ScreenSize');
                screenSize = screenPixels(3:4);
                figurePixels = GtBaseGuiElem.getPixels(obj.conf.h_figure, 'Position');
                figureSize = figurePixels(3:4);
                diffPixels = screenSize - figureSize;
                indexes = diffPixels < 0;
                newFigurePixels = figureSize;
                newFigurePixels(indexes) = screenSize(indexes);
                indexes = diffPixels > (screenSize/2);
                newFigurePixels(indexes) = screenSize(indexes) / 2;
                GtBaseGuiElem.setPixels(obj.conf.h_figure, 'Position', [figurePixels(1:2) newFigurePixels]);
            else
                obj.conf.h_figure = gtGetParentFigure(obj.conf.parent);
                obj.conf.currentParent = obj.conf.parent;
            end

            % Gui objects

            % GUI Callbacks
        end

        function addIncludedGuiElem(obj, elem)
            obj.children_gui{end+1} = elem;
        end

        function addTimer(obj, varargin)
            t = timer(varargin{:});
            obj.ui_timers{end+1} = t;
            start(t);
        end

        function guiQuit(obj)
        % If you overload it, please always call it at some point!
            obj.delUICallbacks();

            % Call also the quit functions of the included objects
            for n = 1:length(obj.children_gui)
                delete(obj.children_gui{n});
            end

            % Call also the quit functions of the timers
            for n = 1:length(obj.ui_timers)
                try
                    stop(obj.ui_timers{n});
                    delete(obj.ui_timers{n});
                catch mexc
                    gtPrintException(mexc)
                end
            end

            % If the figure is our, let's close it
            if isempty(obj.conf.parent)
                try
                    delete(obj.conf.h_figure)
                catch mexc
                    gtPrintException(mexc, 'Problem in closing figure');
                end
            end
        end

        function addUICallback(obj, elem, cb_type, cb_function, is_java)
            if exist('is_java', 'var') && is_java
                elem = findjobj(elem, 'Persist');
            end
            set(elem, cb_type, cb_function);
            obj.ui_callbacks{end+1} = {elem, cb_type};
        end

        function delUICallbacks(obj)
            for n = 1:length(obj.ui_callbacks)
                cb = obj.ui_callbacks{n};
                set(cb{1}, cb{2}, '');
            end
        end


        % Utility Functions
        function strParam = toString(~, content)
            if (islogical(content))
                if (content)
                    strParam = 'true';
                else
                    strParam = 'false';
                end
            elseif (isnumeric(content))
                strParam = num2str(content);
            else
                strParam = content;
            end
        end
    end

    methods (Static, Access = public)
        function pixels = getPixels(elem, property)
            units = get(elem, 'Units');
            set(elem, 'Units', 'pixel');
            pixels = get(elem, property);
            set(elem, 'Units', units);
        end

        function setPixels(elem, property, pixels)
            units = get(elem, 'Units');
            set(elem, 'Units', 'pixel');
            set(elem, property, pixels);
            set(elem, 'Units', units);
        end
    end

    methods (Abstract, Access = protected)
        resetUiComponents(obj)
        doUpdateDisplay(obj)
        addUICallbacks(obj)
    end
end
