function [samXYZ, lab2sam] = gtGeoLab2Sam(labXYZ, omega, labgeo, samgeo, ...
                             freevec, rot_flag)
% GTGEOLAB2SAM  LAB -> SAMPLE reference coordinate transformation
%
% [samXYZ, lab2sam] = gtGeoLab2Sam(labXYZ, omega, labgeo, ...
%                                  samgeo, freevec, rot_flag)
%
% ----------------------------------------------------------------------
%
% Given the LAB coordinates of a set of points, transforms them 
% into SAMPLE coordinates according to an omega rotation. Omega is usually
% positive.
% It optionally creates the transformation matrix.
%
% INPUT  
%   labXYZ         - coordinates in the Lab reference in pixels (n,3)
%   omega          - omega angles in degrees (n,1)
%   labgeo.rotpos  - rotation axis position in Lab coordinates (1,3)
%   labgeo.rotdir  - rotation axis direction (right-handed rotation)
%   samgeo.orig    - origin of the sample reference in Lab coordinates (1,3)
%   samgeo.voxsize - [x,y,z] voxel sizes in lab units
%   samgeo.lab2sam - optional (can be left missing or empty); transformation
%                    matrix from Lab to Sample coordinates for row vectors
%                    (3,3)
%   freevec        - 0 for fixed vectors (e.g. location);
%                    1 for free vectors (e.g. orientation)
%   rot_flag       - if true, only applies rotation so the norm of the 
%                    input vectors will be unchanged; 
%                    i.e. no scaling applied according to the 
%                    voxel sizes (only works if 'samgeo.lab2sam' is undefined)
%
% OUTPUT 
%   samXYZ         - coordinates in the Sample reference in no. of voxels (n,3)
%   lab2sam        - coordinate transformation matrix
%


n = size(labXYZ,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Change of reference from Lab to Sample
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Transformation matrix from Lab to Sample coordinates:
%   Note that Lab is an orthonormal basis by definition, but Sample may not be !!
%   Therefore needs calculation of the inverse, and not only transpose.
if (~isfield(samgeo,'lab2sam') || (isfield(samgeo,'lab2sam') && isempty(samgeo.lab2sam)))
    
    if rot_flag
        % Keep norm of input vectors -> keep unit lengths in the rows of 
        % the transformation matrix 
        sam2lab = [samgeo.dirx; samgeo.diry; samgeo.dirz];
    else
        sam2lab = [samgeo.dirx*samgeo.voxsize(1);...
                   samgeo.diry*samgeo.voxsize(2);...
                   samgeo.dirz*samgeo.voxsize(3)];
    end
    
    lab2sam = inv(sam2lab);

elseif (exist('rot_flag', 'var') && rot_flag)
    error('Transformation matrix already defined. Set ''rot_flag'' to false.')
else
    lab2sam = samgeo.lab2sam;   
end

%%%%%%%%%%%%%%%%%%%%%
%% Rotation in Lab
%%%%%%%%%%%%%%%%%%%%%

% Tensor which rotates with omega from Lab back to Sample:
rotcomp    = gtMathsRotationMatrixComp(labgeo.rotdir, 'row');
rottensors = gtMathsRotationTensor(-omega, rotcomp);


if freevec  % free vectors
    
    % Multiply element-wise
    pro = reshape([labXYZ, labXYZ, labXYZ]',3,n*3).*reshape(rottensors,3,n*3);

    % Sum, reshape, transpose to get rotated vector
    labXYZ_rot = reshape(sum(pro,1),3,n)';

    % Change ref. base
    samXYZ = labXYZ_rot*lab2sam;

else % offset vectors
    
    % Vectors relative to rotation axis position
    labXYZrel = labXYZ - labgeo.rotpos(ones(n,1),:);
    
    % Apply rottensor element-wise
    pro = reshape([labXYZrel, labXYZrel, labXYZrel]',3,n*3).*reshape(rottensors,3,n*3);

    % Sum, reshape, transpose, set offset back to get rotated vector
    labXYZ_rot = reshape(sum(pro,1),3,n)' + labgeo.rotpos(ones(n,1),:);
 
    % Change ref. base
    samXYZ = (labXYZ_rot - samgeo.orig(ones(n,1),:))*lab2sam;
    
end


end % of function
