function labgeo = gtGeoLabDefaultParameters(acq)
% GTGEOLABDEFAULTPARAMETERS
%     labgeo = gtGeoLabDefaultParameters(acq)
%     ---------------------------------------
%     Use of acq.dist, acq.pixelsize, acq.xdet, acq.ydet,
%     acq.no_direct_beam, acq.bb, acq.rotation_axis, acq.rotation_direction,
%     acq.detector_definition
% 

labgeo = [];
switch (acq.rotation_axis)
    case 'vertical'
        % It assumes clockwise rotation
        labgeo.rotdir = [0 0 -1];
    case 'horizontal'
        labgeo.rotdir = [0 -1 0];
    otherwise
        gtError('gtGeoLabDefaultPArameters:wrongArgument', ...
            'Wrong rotation_axis: "%s"', acq.rotation_axis)
end

switch (acq.rotation_direction)
    case 'clockwise'
        % It assumes clockwise rotation
    case 'counterclockwise'
        labgeo.rotdir = -1 * labgeo.rotdir;
    otherwise
        gtError('gtGeoLabDefaultPArameters:wrongArgument', ...
            'Wrong rotation_direction: "%s"', acq.rotation_direction)
end
    
if (acq.no_direct_beam && strcmp(acq.detector_definition, 'vertical'))
    % Assume vertical setup; detector is above sample pointing downwards (-Z)
    disp(['No direct beam case with a vetical detector. ' ...
        'Any flip was applied during the scan... Please check if it is true'])
    labgeo.detrefpos = [0 acq.dist 0];
    labgeo.detdiru   = [0 0 -1];
    labgeo.detdirv   = [1 0 0];
    if (strcmp(acq.rotation_direction, 'counterclockwise'))
        % is this really true?
        labgeo.detrefpos = -1 * labgeo.detrefpos;
        labgeo.detdiru   = -1 * labgeo.detdiru;
    end
    if (isfield(acq, 'bb') && isempty(acq.bb))
        % guess a sample bounding box
        acq.bb = [1 1 4 4];
        disp('Choose a ''fake'' squared sample bounding box in the left-top corner of the image...')
    end
    % Should guess sample envelope
    labgeo.samenvtop = [];
    labgeo.samenvbot = [];
    labgeo.samenvrad = [];
elseif (~acq.no_direct_beam)
    % inline detector HR camera - suppose it on x-axis
    disp('High resolution direct beam scan...')
    
    labgeo.detrefpos = [acq.dist 0 0];
    labgeo.detdiru   = [0  1  0];
    labgeo.detdirv   = [0  0 -1];
    if (strcmp(acq.rotation_direction, 'counterclockwise'))
        % is this really true?
        labgeo.detdiru = -1 * labgeo.detdiru;
        labgeo.detdirv = -1 * labgeo.detdirv;
    end
    % Should take sample envelope from sample bbox (parameters.acq.bb) if
    % exists, otherwise in gtPreprocessing
    if (isfield(acq, 'bb') && ~isempty(acq.bb))
        labgeo.samenvtop = (acq.bb(4)/2) * acq.pixelsize * 1.1;
        labgeo.samenvbot = -labgeo.samenvtop;
        labgeo.samenvrad = (acq.bb(3)/2) * acq.pixelsize * 1.1;
    else
        labgeo.samenvtop = [];
        labgeo.samenvbot = [];
        labgeo.samenvrad = [];
    end
elseif (acq.no_direct_beam && strcmp(acq.detector_definition, 'inline'))
    % inline detector - suppose it on x-axis
    % taper scan (far field)
    disp('This looks like a taper scan...')
    labgeo.detrefpos = [acq.dist 0 0];
    labgeo.detdiru   = [0 1 0];
    labgeo.detdirv   = [0 0 -1];

    % Should take sample envelope from sample bbox (parameters.acq.bb) if
    % exists, otherwise in gtPreprocessing
    if (isfield(acq,'bb') && isempty(acq.bb))
        % guess a sample bounding box
        acq.bb = [acq.xdet/2 -50 acq.ydet/2-50 100 100];
    end
    labgeo.samenvtop = (acq.bb(4)/2) * acq.pixelsize * 1.1;
    labgeo.samenvbot = -labgeo.samenvtop;
    labgeo.samenvrad = (acq.bb(3)/2) * acq.pixelsize * 1.1;
    disp('Choose a squared sample bounding box in the center of the image...')
end

labgeo.pixelsizeu = acq.pixelsize;
labgeo.pixelsizev = acq.pixelsize;
labgeo.detsizeu   = acq.xdet; % number of pixels along the u direction
labgeo.detsizev   = acq.ydet; % number of pixels along the v direction

% Beam direction, rot. axis position and detector properties
labgeo.beamdir = [1 0 0];
labgeo.rotpos  = [0 0 0];
labgeo.detrefu = (labgeo.detsizeu / 2) + 0.5;
labgeo.detrefv = (labgeo.detsizev / 2) + 0.5;

% Coordinate axes definition and Lab unit (for records only)
labgeo.deflabX = 'Along the beam direction.';
labgeo.deflabY = 'Right-handed from Y=cross(Z,X).';
labgeo.deflabZ = 'Along rotation axis. Positive away from sample stage.';
labgeo.labunit = 'mm';
        
% labgeo.detorig is redundant, and can be calculated any time; so better not to have
% it here.
% labgeo.detnorm is redundant, and can be calculated any time; so better not to have
% it here.


end % end of function
