function gtBatch(action, varargin)
% GTBATCH
%     gtBatch((action, varargin)
%     ----------------------------
%
%     Check OAR arrayjobs from OAR and update oarparameters.[xml|mat]
%
%     INPUT:
%       action   = { 'update' } : updates fields and shows them
%                   | 'delete'  : deletes entries and oar jobs
%                   | 'clean'   : cleans entries in file (implied by 'update')
%                   | 'view'    : calls gtBetchLogView (disabled for now)
%
%       varargin = 'name'       : expects a string which is a regular expression
%                   | 'array'   : expects a list of array ids
%                   | 'job'     : expects a list of job ids
%                   | <string>  : implies 'name'
%                   | [<double>]: implies 'job'
%                   | {nothing} : implies 'name' with string 'gt*'

    if (~exist('action', 'var'))
        action = 'update';
    end

    oarparams = gtOarLoadParameters('oarparameters');
    
    switch(lower(action))
        case 'update'
            [records, ~] = getRecords(oarparams, varargin);
            oarparams = gtBatchUpdate(oarparams, records);
            oarparams = gtBatchClean(oarparams);
            gtOarSaveParameters('oarparameters', oarparams);
        case 'clean'
            oarparams = gtBatchClean(oarparams);
            gtOarSaveParameters('oarparameters', oarparams);
        case 'delete'
            [records, selectionType] = getRecords(oarparams, varargin);
            oarparams = gtBatchDelete(records, selectionType);
            oarparams = gtBatchClean(oarparams);
            gtOarSaveParameters('oarparameters', oarparams);
%         case 'view'
%             gtBatchLogview(varargin);
        case 'submit'
            oarparams = gtBatchSubmit(oarparms, varargin);
            gtOarSaveParameters('oarparameters', oarparams);
        otherwise
            gtError('gtBatch:wrong_argument', 'Action %s is not valid!', action)
    end
end

function oarparams = gtBatchUpdate(oarparams, records)
    vars = getVars();
    njobs = numel(records);
    fprintf('Quering OAR to get fresh information...')
    newOarStats = gtOarGetStats([records(:).jobId], vars);
    fprintf(' Done.\n')

    %%%%%%%%%%%%%%%%%%%%%%%
    % loop through jobs
    for m = 1 : njobs
        jobId    = records(m).('jobId');
        arrayid  = records(m).('array_id');
        funcName = records(m).('function');

        % Checking presence of fields
        if (~hasAllFields(newOarStats{m}, vars))
            fprintf('Skipping incomplete record for Job: %d (consider deleting it)\n', jobId)
            continue
        end

        h_name = length(funcName)+2;
        h_job  = length(num2str(jobId))+2;
        h_other = length(datestr(now(),'yyyy-mm-dd HH:MM:SS'))+2;

        arrayName = sprintf('array%d', arrayid);
        jobs = oarparams.(funcName).(arrayName).job;
        indx = find([jobs(:).jobId] == jobId, 1);

        fprintf('%*s %*d ', h_name, funcName, h_job, records(m).jobId)
        for l = 1:length(vars)
            temp = newOarStats{m}.(vars{l});
            jobs(indx).(vars{l}) = temp;
            fprintf('%*s ', h_other, records(m).(vars{l}))
        end
        fprintf('\n')
        oarparams.(funcName).(arrayName).job = jobs;
    end
end

function oarparams = gtBatchClean(oarparams)
    fnames = fieldnames(oarparams);
    for fname = fnames'
        funcRecord = oarparams.(fname{1});
        arrayNames = fieldnames(funcRecord);
        for arrayName = arrayNames'
            arrayRecord = funcRecord.(arrayName{1});
            for n = 1:numel(arrayRecord.job)
                jobs = arrayRecord.job(n);
                states = {jobs(:).state};
                codes = {jobs(:).exit_code};
                matchesStates = strcmp(states, 'Terminated');
                matchesCodes = strcmp(codes, '0 (0,0,0)');
                matches = matchesStates | matchesCodes;
                arrayRecord.job(n) = jobs(matches);
            end
            if (isempty(arrayRecord.job))
                funcRecord = rmfield(funcRecord, arrayName{1});
                fprintf('All the jobs from array %s are terminated with exit_code ''0 (0,0,0)''\n', ...
                    arrayName{1}(6:end))
            end
        end
        if (isempty(fieldnames(funcRecord)))
            oarparams = rmfield(oarparams, fname{1});
            fprintf('All arrays of function %s are terminated with exit_code ''0 (0,0,0)''\n', ...
                fname{1})
        end
    end
end

function oarparams = gtBatchDelete(oarparams, records, selectionType)
    switch(selectionType)
        case 'name'
            funcNames = unique({records(:).function});
            for fname = reshape(funcNames, [1 numel(funcNames)])
                questionStr = sprintf(['Are you sure to remove function ' ...
                    '"%s", from OAR and oarparams.mat? [y/n]'], fname{1});
                check = inputwdefault(questionStr, 'y');
                if strcmpi(check, 'y')
                    fprintf('Deleting jobs from OAR... ')
                    arrayNames = fieldnames(oarparams.(fname{1}));
                    for arrayName = reshape(arrayNames, [1 numel(arrayNames)])
                        deleteOarArray(arrayName{1}(6:end));
                    end
                    fprintf('Done.\nRemoving information about the jobs... ')
                    oarparams = rmfield(oarparams, fname{1});
                    fprintf('Done.\n')
                end
            end
        case 'array'
            arrayIds = unique(records(:).array_id);
            for array_id = reshape(arrayIds, [1 numel(arrayIds)])
                questionStr = sprintf(['Are you sure to remove array ' ...
                    '"%d", from OAR and oarparams.mat? [y/n]'], array_id);
                check = inputwdefault(questionStr, 'y');
                if strcmpi(check, 'y')
                    fprintf('Deleting jobs from OAR... ')
                    deleteOarArray(array_id);
                    fprintf('Done.\nRemoving information about the jobs... ')
                    funcName = records(find(records(:).array_id == array_id, 1)).function;
                    arrayName = sprintf('array%s', array_id);
                    oarparams.(funcName) = rmfield(oarparams.(funcName), arrayName);
                    fprintf('Done.\n')
                end
            end
        case 'job'
            gtError('gtBatch:wrong_argument', ...
                'OAR doesn''t support deletion of single jobs')
    end
end

function oarparams = gtBatchSubmit(oarparms, args)
end

function hasFields = hasAllFields(structToCheck, fields)
    for n = 1:numel(fields)
        if (~isfield(structToCheck, fields{n}))
            hasFields = false;
            return
        end
    end
    hasFields = true;
end

function deleteOarArray(array)
% DELETEOARARRAY
% deleteOarArray(array)

    if (~ischar(array))
        array = num2str(array);
    end
    [~, message] = unix(['oardel --array ' array]);

    if (strfind(message, '[ERROR]'))
        error('gtBatch:OAR:error_while_deleting', message);
    end
end

function [records, selectionType] = getRecords(oarparams, args)
    possibleSelections = {'name', 'array', 'job'};
    switch(numel(args))
        case 0
            selectionType = 'name';
            selectionValue = 'gt*';
        case 1
            if (ischar(args{1}))
                if (ismember(args{1}, possibleSelections))
                    gtError('gtBatch:wrong_argument', 'Not enough arguments');
                else
                    selectionType = 'name';
                    selectionValue = args{1};
                end
            else
                selectionType = 'job';
                selectionValue = args{1};
            end
        case 2
            if ( (~ischar(args{1})) ...
                    || (ischar(args{1}) && ~ismember(lower(args{1}), possibleSelections)) )
                gtError('gtBatch:wrong_argument', 'First argument should be a string');
            end
            selectionType = lower(args{1});
            selectionValue = args{2};
        otherwise
            gtError('gtBatch:wrong_argument', 'Too many arguments');
    end

    switch(lower(selectionType))
        case 'name'
            records = getJobRecordsByName(oarparams, selectionValue);
        case 'array'
            records = getJobRecordsByArrayIDs(oarparams, selectionValue);
        case 'job'
            records = getJobRecordsByJobIDs(oarparams, selectionValue);
    end
end

function records = getJobRecordsByName(oarparams, funcRegexp)
    records = makeEmptyRecord();
    fnames = fieldnames(oarparams);
    % Queryname will be a regular expression we can use to match function names
    matches = regexp(fnames, funcRegexp, 'once');

    for funcNum = 1:numel(matches)
        if (~isempty(matches{funcNum}))
            funcName = fnames{funcNum};
            arrayIds = fieldnames(oarparams.(funcName));
            for arrayIdName = arrayIds'
                arrayId = str2double(arrayIdName{1}(6:end));
                arrayField = oarparams.(funcName).(arrayIdName{1});
                for n = 1:numel(arrayField.job)
                    job = arrayField.job(n);
                    records(end+1) = makeRecord(funcName, arrayId, job.jobId, ...
                        job.state, job.exit_code, job.submissionTime, ...
                        job.startTime, job.stopTime);
                end
            end
        end
    end
end

function records = getJobRecordsByArrayIDs(oarparams, arrayIds)
    records = makeEmptyRecord();
    fnames = fieldnames(oarparams);

    for funcNum = 1:numel(fnames)
        funcName = fnames{funcNum};
        funcArrayIds = fieldnames(oarparams.(funcName));
        for arrayIdName = funcArrayIds'
            arrayId = str2double(arrayIdName{1}(6:end));
            if (ismember(arrayId, arrayIds))
                arrayField = oarparams.(funcName).(arrayIdName{1});
                for n = 1:numel(arrayField.job)
                    job = arrayField.job(n);
                    records(end+1) = makeRecord(funcName, arrayId, job.jobId, ...
                        job.state, job.exit_code, job.submissionTime, ...
                        job.startTime, job.stopTime);
                end
            end
        end
    end
end

function records = getJobRecordsByJobIDs(oarparams, jobIds)
    records = makeEmptyRecord();
    fnames = fieldnames(oarparams);

    for funcNum = 1:numel(fnames)
        funcName = fnames{funcNum};
        arrayIds = fieldnames(oarparams.(funcName));
        for arrayIdName = arrayIds'
            arrayId = str2double(arrayIdName{1}(6:end));
            arrayField = oarparams.(funcName).(arrayIdName{1});
            for n = 1:numel(arrayField.job)
                job = arrayField.job(n);
                if (ismember(job.jobId, jobIds))
                    records(end+1) = makeRecord(funcName, arrayId, job.jobId, ...
                        job.state, job.exit_code, job.submissionTime, ...
                        job.startTime, job.stopTime);
                end
            end
        end
    end
end

function record = makeEmptyRecord()
    record = makeRecord({}, {}, {}, {}, {}, {}, {}, {});
end

function record = makeRecord(func, array, jobId, state, exitCode, submissionTime, startTime, stopTime)
    vars = getVars();
    vars = [{'function', 'array_id', 'jobId'} vars{:}];
    vars(2, :) = {func, array, jobId, state, exitCode, submissionTime, startTime, stopTime};
    record = struct(vars{:});
end

function vars = getVars()
    vars = {'state', 'exit_code', 'submissionTime', 'startTime', 'stopTime'};
end
